"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class AlgorithmBase {
    constructor() {
        // eslint-disable-next-line no-magic-numbers
        this._generationNumber = 0;
        // eslint-disable-next-line no-magic-numbers
        this._offspringNumber = 0;
        this._initialized = false;
    }
    get generationNumber() {
        return this._generationNumber;
    }
    get offspringNumber() {
        return this._offspringNumber;
    }
    get initialized() {
        return this._initialized;
    }
    get best() {
        return this.population.best;
    }
    get progress() {
        return this.termination.progress;
    }
    get hasReached() {
        return this.termination.hasReached(this);
    }
    reset() {
        return __awaiter(this, void 0, void 0, function* () {
            this.population.init();
            yield Promise.all(this.population.chromosomes.map((chromosome) => __awaiter(this, void 0, void 0, function* () {
                yield this.fitness.evaluate(chromosome);
            })));
            this.termination.init();
            this._generationNumber = 0;
            this._offspringNumber = 0;
            this.performReset();
            this._initialized = true;
        });
    }
    performReset() {
        // override if required
    }
    step() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.hasReached) {
                return;
            }
            if (!this._initialized) {
                this.reset();
            }
            const { parents, population } = yield this.selection.select(this.population.chromosomes);
            const offspring = yield this.crossover.cross(parents, this.crossoverProbability);
            yield Promise.all(offspring.map((chromosome) => __awaiter(this, void 0, void 0, function* () {
                yield this.mutation.mutate(chromosome, this.mutationProbability);
                yield this.fitness.evaluate(chromosome);
            })));
            yield Promise.all(parents.map((chromosome) => __awaiter(this, void 0, void 0, function* () {
                if (chromosome.fitness === undefined) {
                    yield this.fitness.evaluate(chromosome);
                }
            })));
            const newGeneration = yield this.reinsertion.select(population, offspring, parents, this.population.size);
            yield this.population.update(newGeneration);
            this._generationNumber++;
            this._offspringNumber += offspring.length;
            this.performStep();
        });
    }
    performStep() {
        // override if required
    }
}
exports.AlgorithmBase = AlgorithmBase;
//# sourceMappingURL=AlgorithmBase.js.map